module vga( input clk108,			
			   output reg hsync,		// horizontal sync
			   output reg vsync,		// vertical sync
			   output reg hstart,		// line start on next clock
			   output reg vstart,		// field starts on next clock
			   output reg hblank = 1,	// hblank on next clock
			   output reg vblank = 0	// vblank on next clock
			   );
// timing parameters
parameter 
    HFRONT  = 16,		// front porch (+right border)
    HVIDEO  = 320,		// active video area 
    HSYNC   = 32,		// hsync 
    HBACK   = 48;		// back porch  (+left border)

parameter
    VFRONT  = 7,		// front porch (+bottom border)
    VVIDEO  = 200,		// active vide area
    VSYNC   = 2,		// vsync
    VBACK   = 15;  // back porch  (+top border)

// states (used for both H and V state machines)
parameter
    VIDEO  = 2'd0,
    FRONT  = 2'd1,
    SYNC   = 2'd2,
    BACK   = 2'd3;

// horizontal counters and state
reg [9:0] hcount = 0;		// down counter for horizontal state 
reg [8:0] hload;		// next load value
reg [1:0] hstate = FRONT;	// horizontal state
wire hcount_done = hcount[9];	// done when count < 0

// vertical counters and state
reg [8:0] vcount = 0;		// down counter for vertical state
reg [7:0] vload;		// next load value
reg [1:0] vstate = FRONT;	// horizontal state
wire vcount_done = vcount[8];	// done when count < 0

// enable signal for V state machine
wire venable = (hstate == FRONT) & hcount_done;

// preload next timer value depending on state
// subtract 2 from timing value to compensate for pipeline delays
// This should be synthesized as a small LUT ROM, so it's quite
// efficient.
always @(posedge clk108)
	hload <= (hstate == BACK)  ? HVIDEO - 2:
	         (hstate == VIDEO) ? HFRONT - 2:
		 (hstate == FRONT) ? HSYNC - 2:
		  		     HBACK - 2;

// do the same for vertical 
always @(posedge clk108)
	vload <= (vstate == BACK)  ? VVIDEO - 2:
	         (vstate == VIDEO) ? VFRONT - 2:
		 (vstate == FRONT) ? VSYNC  - 2:
		  		     VBACK  - 2;

// adjust down counter. Reload when it's done.
always @(posedge clk108) 
	     if( hcount_done )
		  hcount <= { 1'b0, hload };
	     else
		  hcount <= hcount - 1;

// when down counter is done, go to next state
always @(posedge clk108)
	if( hcount_done ) 
	    case( hstate )
		    VIDEO : hstate <= FRONT;
		    FRONT : hstate <= SYNC;
		    SYNC  : hstate <= BACK;
		    BACK  : hstate <= VIDEO;
	    endcase

// start of active line
always @(posedge clk108)
	    hstart <= (hstate == BACK) & hcount_done & ~vblank;

// vstart pulse to signal end of vertical blanking
always @(posedge clk108)
	    vstart <= (vstate == BACK) & venable & vcount_done;

// adjust down counter. Reload when it's done.
always @(posedge clk108) 
	if( venable ) begin
	     if( vcount_done )
		  vcount <= { 1'b0, vload };
	     else
		  vcount <= vcount - 1;
	end

// when down counter is done, go to next state
always @(posedge clk108)
	if( venable & vcount_done ) 
	    case( vstate )
		    VIDEO : vstate <= FRONT;
		    FRONT : vstate <= SYNC;
		    SYNC  : vstate <= BACK;
		    BACK  : vstate <= VIDEO;
	    endcase

// update registered outputs
always @(posedge clk108) begin
	    hsync  <= hstate == SYNC;
	    vsync  <= vstate == SYNC;
	    hblank <= (vstate == VIDEO) & (hstate == VIDEO) & hcount_done;
	    vblank <= vstate != VIDEO;
	end
endmodule